{% extends "base.html" %}

{% block title %}
	&raquo; Non-core Contracts &raquo; {{ rate_script.contract.name }} &raquo; 
 	{{ rate_script.start_date|date_format }}
{% endblock %}

{% block nav %}
	<a href="/non_core_contracts">Non-core Contracts</a> &raquo; 
	<a href="/non_core_contracts/{{rate_script.contract.id}}"
		>{{rate_script.contract.name}}</a> &raquo;
 {{ rate_script.start_date|date_format }} [<a
	href="/non_core_rate_scripts/{{rate_script.id}}/edit">edit</a>]
{% endblock %}

{% block content %}
	<table class="DataTable widthAuto noMarginBottom">
		<caption>Rate Script</caption>	
		<tr>
			<th>Start Date</th>
			<td>{{ rate_script.start_date|date_format }}</td>
		</tr>
		<tr>
			<th>Finish Date</th>
			<td>{{ rate_script.finish_date|date_format }}</td>
		</tr>
		<tr>
			<td>
				{% if previous_rate_script %}
					<a
						href="/non_core_rate_scripts/{{previous_rate_script.id}}"
						>Previous Rate Script</a>
				{% else %}
					No Previous Rate Script
				{% endif %}
			</td>
			<td>
				{% if next_rate_script %}
					<a
						href="/non_core_rate_scripts/{{next_rate_script.id}}"
						>Next Rate Script</a>
				{% else %}
					No Next Rate Script
				{% endif %}
			</td>
		</tr>
	</table>

	<h3>Script</h3>

	<pre>{{ rate_script.script }}</pre>
{% endblock %}
