{% extends "base.html" %}

{% block title %}
	&raquo; Profile Classes &raquo; {{pc.code}}
{% endblock %}

{% block nav %}
	<a href="/e/pcs">Profile Classes</a> &raquo; {{pc.code}}
{% endblock %}

{% block content %}
	<table>
		<tr>
			<th>Code</th>
			<td>{{pc.code}}</td>
		</tr>
		<tr>
			<th>Description</th>
			<td>{{pc.name}}</td>
		</tr>
		<tr>
			<th>Valid From</th>
			<td>{{pc.valid_from|date_format}}</td>
		</tr>
		<tr>
			<th>Valid To</th>
			<td>{{pc.valid_to|date_format}}</td>
		</tr>
	</table>
{% endblock %}
