{% extends "base.html" %}

{% block title %}
	&raquo; DTC Meter Types &raquo; {{dtc_meter_type.code}}
{% endblock %}

{% block nav %}
	<a href="/e/dtc_meter_types">DTC Meter Types</a> &raquo; {{dtc_meter_type.code}}
{% endblock %}

{% block content %}

	<table>
		<tr>
			<th>Code</th>
			<td>{{dtc_meter_type.code}}</td>
		</tr>
		<tr>
				<th>Description</th>
				<td>{{dtc_meter_type.description}}</td>
		</tr>
	</table>

{% endblock %}
