{% extends "base.html" %}

{% block title %}
	&raquo; CSV Sites Monthly Duration
{% endblock %}

{% block nav %}
	CSV Sites Monthly Duration
{% endblock %}

{% block content %}

	<form action="/reports/161">
		<fieldset>
			<legend>Download</legend>	
			<label>Months</label> {{input_text('months', '1', 2, 2)}}
			<label>Final Month</label>
			{{input_date('finish', month_finish, resolution='month')}}
			<input type="submit" value="Download">
		</fieldset>
	</form>
	
{% endblock %}
