{% extends "base.html" %}

{% block title %}
	&raquo; CSV Supplies TRIAD
{% endblock %}

{% block nav %}
	CSV Supplies TRIAD
{% endblock %}

{% block content %}

	<form action="/reports/41">
		<legend>Download Supplies TRIAD CSV</legend>	
		<fieldset>
			<label>For financial year ending March</label>
			{{input_text('year', year, 4, 4)}} 
			<input type="submit" value="Download">
		</fieldset>
	</form>

{% endblock %}
