{% extends "base.html" %}

{% block title %}
	&raquo; Glossary
{% endblock %}

{% block nav %}
	Glossary
{% endblock %}

{% block content %}

	<p>{{glossary_intro}}</p>

	<table class="sticky">
		<thead>
			<tr>
				<th>Term</th>
				<th>Description</th>
			</tr>
		</thead>
		<tbody>
			{% for term, desc in glossary_terms|dictsort %}
				<tr>
					<td>{{term}}</td>
					<td>{{desc}}</td>
				</tr>
			{% endfor %}
		</tbody>
		<tbody>
			<tr><th colspan="2">Tariff Elements</th></tr>
			{% for term, desc in glossary_elements|dictsort %}
				<tr>
					<td>{{term}}</td>
					<td>{{desc}}</td>
				</tr>
			{% endfor %}
		</tbody>
	</table>
	
{% endblock %}
