{% extends "base.html" %}

{% block title %}
	&raquo; Meter Types &raquo; {{meter_type.description}}
{% endblock %}

{% block nav %}
	<a href="/e/meter_types">Meter Types</a> &raquo; {{meter_type.description}}
{% endblock %}

{% block content %}

	<table>
		<caption>{{meter_type.description}}</caption>	
		<tr>
			<th>Code</th>
			<td>{{meter_type.code}}</td>
		</tr>
		<tr>
			<th>Description</th>
			<td>{{meter_type.description}}</td>
		</tr>
		<tr>
			<th>Valid From</th>
			<td>{{meter_type.valid_from|date_format}}</td>
		</tr>
		<tr>
			<th>Valid To</th>
			<td>{{meter_type.valid_to|date_format}}</td>
		</tr>
	</table>
	
{% endblock %}
