{% extends "base.html" %}

{% block title %}
	&raquo; DNOs &raquo; {{dno.dno_code}} &raquo;
 	Rate Script {{rate_script.start_date|date_format}}
{% endblock %}

{% block nav %}
	<a href="/e/dnos">DNOs</a> &raquo; 
	<a href="/e/dnos/{{dno.id}}">{{dno.dno_code}}</a> &raquo;
	Rate Script {{rate_script.start_date|date_format}}
	[<a href="/e/dno_rate_scripts/{{rate_script.id}}/edit">edit</a>]
{% endblock %}

{% block content %}

	<table>
		<caption>Rate Script</caption>	
		<tbody>
			<tr>
				<td>Start Date</td>
				<td>{{rate_script.start_date|date_format}}</td>
			</tr>
			<tr>
				<td>Finish Date</td>
				<td>{{rate_script.finish_date|date_format}}</td>
			</tr>
		</tbody>
	</table>

	<h2>Script</h2>	

	<pre>{{rate_script.script}}</pre>

{% endblock %}
