{% extends "base.html" %}

{% block title %}
	&raquo; DNOs
{% endblock %}

{% block nav %}
	DNOs
{% endblock %}

{% block content %}
	<table>
		<thead>
			<tr>
				<th>Code</th>
				<th>Name</th>
				<th>Start Date</th>
				<th>Finish Date</th>
			</tr>
		</thead>
		<tbody>
			{% for dno in dnos %}
				<tr>
					<td><a href="/e/dnos/{{dno.party.id}}">{{dno.party.dno_code}}</a></td>
					<td>{{dno.party.name}}</td>
					<td>{{dno.contract.start_rate_script.start_date|date_format}}</td>
					<td>{{dno.contract.finish_rate_script.finish_date|date_format}}</td>
				</tr>
			{% endfor %}
		</tbody>
	</table>

	<h2>CSV Downloads</h2>	

	<p>
		<a href="/reports/csv_llfcs">LLFCs</a>
	</p>

{% endblock %}
