{% extends "base.html" %}

{% block title %}
	&raquo; Read Types &raquo; {{read_type.code}}
{% endblock %}

{% block nav %}
	<a href="/e/read_types">Read Types</a> &raquo; {{read_type.code}}
{% endblock %}

{% block content %}
	<table>
		<tr>
			<th>Code</th>
			<td>{{read_type.code}}</td>
		</tr>
		<tr>
			<th>Description</th>
			<td>{{read_type.description}}</td>
		</tr>
	</table>
	
{% endblock %}
