{% extends "base.html" %}

{% block title %}
	&raquo; Supplies &raquo; {{supply.id}} &raquo; Virtual Bill
{% endblock %}

{% block nav %}
	<a href="/e/supplies">Supplies</a> &raquo; 
	<a href="/e/supplies/{{supply.id}}">{{supply.id}}</a> &raquo; 
 	Virtual Bill
{% endblock %}

{% block content %}
	<form>
		<fieldset>
			<legend>Show Virtual Bills</legend>	
			<input type="hidden" name="supply_id" value="{{supply.id}}">
			From {{input_date('start', start_date)}} to
			{{input_date('finish', finish_date)}}
			<input type="submit" value="Show">
		</fieldset>
	</form>

	<table>
		<caption>Summary</caption>
		<tr>
			<th>From</th>
			<td>{{start_date|date_format}}</td>
		</tr>
		<tr>
			<th>To</th>
			<td>{{finish_date|date_format}}</td>
		</tr>
		<tr>
			<th>Net GBP</th>
			<td>{{net_gbp}}</td>
		</tr>
	</table>

	<table>
		<caption>Import Supplier Virtual Bill</caption>
		{% for mera in meras %}
			{% if not mera.skip %}
				<thead>
					<tr>
						{% for heading in mera.headings %}
							<th>{{heading}}</th>
						{% endfor %}
					</tr>
				</thead>
			{% endif %}
			<tbody>
				<tr>
					{% for datum in mera.data %}
							<td>{{datum}}</td>
					{% endfor %}
				</tr>
			</tbody>
		{% endfor %}
	</table>

{% endblock %}
