{% extends "base.html" %}

{% block title %}
	&raquo; Meter Payment Types &raquo; {{meter_payment_type.code}}
{% endblock %}

{% block nav %}
	<a href="/e/meter_payment_types/">Meter Payment Types</a> &raquo;
	{{meter_payment_type.description}}
{% endblock %}

{% block content %}

	<table>
		<tr>
			<th>Code</th>
			<td>{{meter_payment_type.code}}</td>
		</tr>
		<tr>
			<th>Description</th>
			<td>{{meter_payment_type.description}}</td>
		</tr>
		<tr>
			<th>Valid From</th>
			<td>{{meter_payment_type.valid_from|date_format}}</td>
		</tr>
		<tr>
			<th>Valid To</th>
			<td>{{meter_payment_type.valid_to|date_format}}</td>
		</tr>
	</table>

{% endblock %}
